"use strict"

const DEBUG_SPAWN_Y=0;

let game;
let frameTime;
let needDraw;

function initGame() {
    game=new Game();
    game.init();
    window.addEventListener("mousedown",
			    ()=>(game.unpause()));
    window.addEventListener("keydown",
			    (e)=>(game.keydown(e)));
    window.addEventListener("keyup",
			    (e)=>(game.keyup(e)));
    
    frameTime=performance.now();
    setInterval(checkForFrames,1000/240)
    needDraw=true;
    requestAnimationFrame(checkForDraw);
}

function checkForFrames() {
    let now=performance.now();
    for(let i=0;i<2;++i) {
	if(frameTime<now) {
	    game.runTick();
	    frameTime+=1000/60;
	}
    } 
    if(frameTime<now) { frameTime=now; }
}

function checkForDraw() {
    if(needDraw) {
	game.draw();
	needDraw=false;
    }
    requestAnimationFrame(checkForDraw);
}

class Game {
    referencePitchNumber=60
    referencePitchX=310
    semitonePixels=20
    despawnMargin=60
    playerMargin=120
    spawnMargin=960
    centerPanX=320
    maxPanPixels=640
    init() {
	this.soundCache={}
	this.obstacles=[]
	this.despawns=[];
	this.audioContext=new AudioContext();
	this.gainNode=new GainNode(this.audioContext)
	this.clipNode=new WaveShaperNode(this.audioContext);
	this.gainNode.connect(this.clipNode);
	this.clipNode.connect(this.audioContext.destination)
	this.spritesheet=document.getElementById("spritesheet");
	this.drawCanvas=document.getElementById("drawCanvas");
	this.drawContext=this.drawCanvas.getContext("2d");
	this.scrollTop=0;
	this.createSpawnList();
	this.spawnOffset=0;
	this.nextSpawnIndex=0;
	this.player=new Player(this.playerMargin+DEBUG_SPAWN_Y);
	this.scrollTop=Math.floor(this.player.y-this.playerMargin);
	this.resolveSpawns();
	this.resolveDespawns(true)
	this.paused=true;
	this.keysHeld={}
    }
    keydown(e) {
	this.keysHeld[e.code]=true
    }
    keyup(e) {
	this.keysHeld[e.code]=false
    }
    unpause() {
	if(this.paused && this.audioContext.state=="suspended") {
	    this.audioContext.resume().then(()=>(this.paused=false));
	}
	else {
	    this.paused=false;
	}
    }
    runTick() {
	if(!this.paused) {
	    this.player.runTick(this,
			       );
	    this.scrollTop=Math.floor(this.player.y-this.playerMargin);
	    this.resolveDespawns();
	    this.resolveSpawns();
	    needDraw=true;
	}
    }
    draw() {
	this.drawContext.fillStyle="#f8f8ff"
	this.drawContext.fillRect(0,0,
				  this.drawCanvas.width,this.drawCanvas.height);
	for(let o of this.obstacles) {
	    if(o.wallLeft) {
		this.drawLeftWall(o);
	    }
	    if(o.wallRight) {
		this.drawRightWall(o);
	    }
	}
	for(let o of this.obstacles) {
	    this.drawObstacle(o);
	}
	this.drawPlayer();
	for(let d of this.despawns) {
	    this.drawDespawn(d);
	}
	if(this.paused) {
	    this.drawContext.drawImage(
		this.spritesheet,
		224,0,192,128,
		32,32,192,128
	    );
	}
    }
    
    drawLeftWall(o)  {
	let x=Math.floor(o.x)-Math.floor(o.width/2);
	let wallCount=0;
	while(x>-64) { x-=o.wallWidth; wallCount+=1; }
	let y=Math.floor(o.y-this.scrollTop)-Math.floor(o.height/2);
	for(let i=0;i<wallCount;++i) {
	    this.drawContext.drawImage(
		this.spritesheet,
		o.wallTexX,o.wallTexY,o.wallWidth,o.height,
		x,y,o.wallWidth,o.height
	    );
	    x+=o.wallWidth;
	}
    }
    drawRightWall(o)  {
	let x=Math.floor(o.x)-Math.floor(o.width/2)+Math.floor(o.width);
	let wallCount=1;
	while(x<704) { x+=o.wallWidth; wallCount+=1; }
	let y=Math.floor(o.y-this.scrollTop)-Math.floor(o.height/2);
	for(let i=0;i<wallCount;++i) {
	    this.drawContext.drawImage(
		this.spritesheet,
		o.wallTexX,o.wallTexY,o.wallWidth,o.height,
		x,y,o.wallWidth,o.height
	    );
	    x-=o.wallWidth;
	}
    }    
    drawPlayer() {
	this.drawContext.lineCap="round"
	this.drawContext.lineWidth=2
	let cy=this.playerMargin+14
	let len=16;

	let drawSki=(x,color)=> {
	    this.drawContext.strokeStyle=color;
	    let cx=Math.floor(this.player.x)+x;
	    this.drawContext.beginPath();
	    this.drawContext.moveTo(
		cx-len*Math.cos(this.player.theta),
		cy+len*Math.sin(this.player.theta)
	    );	
	    this.drawContext.lineTo(
		cx+len*Math.cos(this.player.theta),
		cy-len*Math.sin(this.player.theta)
	    );
	    this.drawContext.stroke();
	}
	
	if(this.player.theta<Math.PI/2) {
	    drawSki(-4,"#000000");
	    drawSki(4,"#0000ff");
	}	
	else {
	    drawSki(4,"#0000ff");	       
	    drawSki(-4,"#000000");
	}
	if(this.player.damaged && 
	   this.player.damageTick%16<8) { return; }
	this.drawContext.drawImage(
	    this.spritesheet,
	    224,160,32,32,
	    Math.floor(this.player.x)-16,this.playerMargin-16,32,32
	);
	
    }
    drawObstacle(o) {
	let x=Math.floor(o.x)-Math.floor(o.width/2);
	let y=Math.floor(o.y-this.scrollTop)-Math.floor(o.height/2);
	this.drawContext.drawImage(
	    this.spritesheet,
	    o.texX,o.texY,o.width,o.height,
	    x,y,o.width,o.height
	);
	/*
	this.drawContext.fillStyle="#000000"
	this.drawContext.fillText(
	    Math.floor(o.y/320)+" "+Math.floor(o.y%320/40),
	    o.x+20,y
	);
	*/
    }
    drawDespawn(d) {
	let effectiveWidth=Math.ceil(d.width*(1-d.age/d.despawnDuration));
	let effectiveHeight=Math.ceil(d.height*(1-d.age/d.despawnDuration));
	if(effectiveWidth>0 && effectiveHeight>0) {
	    let x=Math.floor(d.x)-Math.floor(effectiveWidth/2);
	    let y=this.despawnMargin-Math.floor(effectiveHeight/2);
	    this.drawContext.drawImage(
		this.spritesheet,
		d.texX,d.texY,d.width,d.height,
		x,y,effectiveWidth,effectiveHeight
	    );	    
	}
    }
    resolveDespawns(quiet) {
	for(let d of this.despawns) {
	    d.age+=1;
	    if(d.age>=d.despawnDuration) {
		d.complete=true;
	    }
	}
	this.despawns=this.despawns.filter(d=>(!d.complete));
	for(let o of this.obstacles) {
	    if(Math.floor(o.y)<=this.scrollTop+this.despawnMargin) {
		o.despawn=true;
		o.despawnDuration=Math.floor(o.soundLength*60);
		o.age=0;
		this.despawns.push(o);
		if(!quiet) { this.playObstacleSound(o); }
	    }
	}
	this.obstacles=this.obstacles.filter(o=>(!o.despawn));
    }
    createSpawnList() {
	this.spawnList=[]
	let parsed=new ScoreParser().parse(GAME_SCORE,this);
	this.spawnList=parsed.spawnList;
	this.spawnLoopDistance=parsed.spawnLoopDistance;
    }
    resolveSpawns() {
	for(;;) {
	    let nextSpawn=this.spawnList[this.nextSpawnIndex];
	    let spawnY=nextSpawn.y+this.spawnOffset;
	    if(spawnY<this.scrollTop+this.spawnMargin) {
		this.obstacles.push(
		    new Obstacle(
			nextSpawn.x, spawnY, nextSpawn.obstacleType
		    )
		)
		++this.nextSpawnIndex;
		if(this.nextSpawnIndex>=this.spawnList.length) {
		    this.nextSpawnIndex=0;
		    this.spawnOffset+=this.spawnLoopDistance;
		}
	    }
	    else {
		return;
	    }
	}
    }
    playObstacleSound(o) {
	let soundKey=o.obstacleType+" "+o.x;
	if(!this.soundCache[soundKey]) {
	    this.soundCache[soundKey]=this.renderObstacleSound(o);
	}
	let node=
	    new AudioBufferSourceNode(this.audioContext,
				      {buffer:this.soundCache[soundKey]})
	node.connect(this.gainNode);
	node.start();
    }
    xToPitch(x) {
	return (x-this.referencePitchX)/this.semitonePixels+
	    this.referencePitchNumber;
    }
    pitchToX(pitch) {
	return (pitch-this.referencePitchNumber)*this.semitonePixels+
	    this.referencePitchX;
    }
    renderObstacleSound(o) {
	let length=Math.floor(o.soundLength*96000);
	let buffer=new AudioBuffer({
	    numberOfChannels:2,
	    length:length,
	    sampleRate:96000
	})
	let left=buffer.getChannelData(0)
	let right=buffer.getChannelData(1)
	let basePitch=this.xToPitch(o.x)+o.octaveShift*12;
	let baseFrequency=440*Math.pow(2,(basePitch-69)/12)
	let pan=.5+(o.x-this.centerPanX)/(this.maxPanPixels*2)
	switch(o.soundType) {
	case "pw":
	    {
		let width=o.initialPulseWidth;
		let phase=0;
		let high=true;
		for(let i=0;i<length;++i) {
		    let cents=o.initialDetune+
			(i/length)*(o.finalDetune-o.initialDetune);
		    let frequency=
			baseFrequency*Math.pow(2,cents/1200);
		    phase=phase+frequency;
		    if(phase>=96000) {
			phase=phase%96000;
			high=true;
			width=o.initialPulseWidth+
			    (i/length)*(o.finalPulseWidth-o.initialPulseWidth);
		    }
		    else {
			if(phase>=width*96000) {
			    high=false;
			}
		    }		    
		    let volume=o.initialVolume+
			(i/length)*(o.finalVolume-o.initialVolume);
		    let signal=high?volume:-volume;
		    left[i]=signal*(1-pan);
		    right[i]=signal*pan;
		}
	    }
	}
	return buffer;
    }
}

class Obstacle {
    constructor(x,y,obstacleType) {
	for(let k in OBSTACLE_TYPES[obstacleType]) {
	    this[k]=OBSTACLE_TYPES[obstacleType][k];
	}
	this.obstacleType=obstacleType;
	this.x=x;
	this.y=y;
    }
}


const OBSTACLE_TYPES={
    "tree1": {
	soundType:"pw",
	initialPulseWidth:.5,
	finalPulseWidth:0,
	soundLength:.5,
	initialVolume:.25,
	finalVolume:.125,
	initialDetune:0,
	finalDetune:-20,
	octaveShift:-2,
	width:32,
	height:32,
	texX:0,
	texY:0
    },
    "tree2": {
	soundType:"pw",
	initialPulseWidth:.5,
	finalPulseWidth:0,
	soundLength:.25,
	initialVolume:.25,
	finalVolume:.125,	
	initialDetune:0,
	finalDetune:-20,
	octaveShift:1,
	width:32,
	height:32,
	texX:64,
	texY:0
    },
    "snowman": {
	soundType:"pw",
	initialPulseWidth:.75,
	finalPulseWidth:0,
	soundLength:.75,
	initialVolume:.25,
	finalVolume:.125,	
	initialDetune:0,
	finalDetune:-20,
	octaveShift:-1,
	width:32,
	height:32,
	texX:32,
	texY:0
    },    
    "trainL": {
	soundType:"pw",
	initialPulseWidth:.25,
	finalPulseWidth:.25,
	soundLength:.5,
	initialVolume:.25,
	finalVolume:.125,
	initialDetune:0,
	finalDetune:-20,
	octaveShift:0,
	width:32,
	wallWidth:32,
	height:32,
	texX:0,
	texY:64,
	wallTexX:32,
	wallTexY:64,
	wallRight:true
    },
    "trainR": {
	soundType:"pw",
	initialPulseWidth:.25,
	finalPulseWidth:.25,
	soundLength:.5,
	initialVolume:.25,
	finalVolume:.125,
	initialDetune:0,
	finalDetune:-20,
	octaveShift:0,
	width:32,
	wallWidth:32,
	height:32,
	texX:32,
	texY:32,
	wallTexX:0,
	wallTexY:32,
	wallLeft:true
    },
}

class Player {
    constructor(y) {
	this.x=320;
	this.y=y;
	this.theta=Math.PI*3/4;
	this.speed=0;
    }
    runTick(game) {
	if(this.damaged) {
	    let xStep=20, yStep=10
	    ++this.damageTick;
	    if(this.x<320-xStep) { this.x+=xStep; }
	    else if(this.x>320+xStep) { this.x-=xStep; }
	    else { this.x=320; }
	    if(this.y>this.returnY+yStep) { this.y-=yStep; }
	    else { this.y=this.returnY; }
	    if(this.x==320 && this.y==this.returnY) {
		this.speed=1;
		this.damaged=false;
	    }
	}
	let turn=(game.keysHeld["ArrowLeft"]?-1:0)+
	    (game.keysHeld["ArrowRight"]?1:0);
	this.theta+=turn/16;
	if(this.theta>3) { this.theta=3; }
	if(this.theta<Math.PI-3) { this.theta=Math.PI-3; }
	if(this.damaged) { return; }
	let targetSpeed=Math.max(0,Math.sin(this.theta)*16-4);
	if(targetSpeed>this.speed) {
	    this.speed+=(targetSpeed-this.speed)/1024;
	}
	else {
	    this.speed+=(targetSpeed-this.speed)/32;
	}
	this.x-=this.speed*Math.cos(this.theta);
	this.y+=this.speed*Math.sin(this.theta);
	if(this.x<0 || this.x>640 || this.colliding(game)) {
	    this.damaged=true;
	    this.damageTick=0;
	    this.returnY=this.y-320;
	}
    }
    colliding(game) {	
	for(let o of game.obstacles) {
	    if(this.y+16>o.y &&
	       this.y<o.y+o.height/2 &&
	       (o.wallRight || this.x<o.x+o.width/2) &&
	       (o.wallLeft || this.x>o.x-o.width/2)) {
		return true;
	    }
	}
	return false;
    }
}

class ScoreParser {
    parse(scoreText,game) {
	let spawnList=[]
	let channels=scoreText.split("$");
	let obstacleTypes=["tree1","tree2","snowman","trainL","trainR"]
	let maxPosition=0;
	for(let channelNum in channels) {
	    let channelScore=channels[channelNum];
	    let position=0;
	    let recentNote=null;
	    let space=()=> {
		recentNote=null;
		position+=1;
		maxPosition=Math.max(position,maxPosition);
	    }
	    let backspace=()=>{ position-=1; }
	    let note=(p)=> {
		recentNote={
		    x:game.referencePitchX+(p-12)*20,
		    y:position*40,
		    obstacleType:obstacleTypes[channelNum]
		}
		spawnList.push(recentNote)
		position+=1;
		maxPosition=Math.max(position,maxPosition);
	    }
	    let change=(c)=>{
		if(recentNote) {
		    recentNote.x+=c*20;
		}
	    }	    
	    for(let character of channelScore) {
		switch(character) {
		case ".": space(); break;
		case "C": note(0); break;
		case "D": note(2); break;
		case "E": note(4); break;
		case "F": note(5); break;
		case "G": note(7); break;
		case "A": note(9); break;
		case "B": note(11); break;
		case "c": note(12); break;
		case "d": note(14); break;
		case "e": note(16); break;
		case "f": note(17); break;
		case "g": note(19); break;
		case "a": note(21); break;
		case "b": note(23); break;
		case "#": change(1); break;
		case "_": change(-1); break;
		case "'": change(12); break;
		case ",": change(-12); break;
		case "+": backspace(); break;
		}
	    }
	}
	spawnList.sort((a,b)=> a.y-b.y);
	return {spawnList:spawnList, spawnLoopDistance:maxPosition*40}
    }
    
}

// very hardcodey syntax/logic here, edit at own risk
const GAME_SCORE=`
.. .. .. ..

C. C. C. C.
C. C. C. C.
G. G. G. G.
G. G. G. G.
d. d. d. d.
d. d. d. B_.
G. G. G. G.
G. G. G. G.

C. C. C. C.
C. C. C. C.
G. G. G. G.
G. G. G. G.
d. d. d. d.
d. d. d. B_.
G. G. G. E_.
G. G. G. G.

C. E_. C. E_.
G. B_. G. B_.
G. B_. d. B_.
G. E_. G. E_.
C. E_. E_. E_.
G. B_. G. B_.
G. B_. G. B_.
G. E_. G. E_.

C. E_. C. E_.
G. B_. G. B_.
G. B_. d. B_.
G. E_. G. E_.
C. E_. C. E_.
G. B_. G. B_.
d. B_. d. B_.
G. E_. G. E_.

c. c. G.  G.
A_. A_. E_. E_.
D. E_. F. G.
E_. G. B_. c.
G. G. A_. A_.
G. G. F. F.
C. C. D. D.
C. D. E_. G.

cc' cc' Gg  Gg
A_a_ A_a_ E_e_ E_e_
Dd E_e_ Ff Gg
E_e_ Gg B_b_ cc'
Gg Gg A_a_ A_a_
Gg Gg Ff Ff
Cc Cc Dd Dd
Cc Dd E_e_ Gg

cc' cc' Gg  Gg
A_a_ A_a_ E_e_ E_e_
Dd E_e_ Ff Gg
E_e_ Gg B_b_ cc'
Gg Gg A_a_ A_a_
Gg Gg Ff Ff
Cc Cc Dd Dd
Cc Dd E_e_ Gg

cc' cc' Gg  Gg
A_a_ A_a_ E_e_ E_e_
Dd E_e_ Ff Gg
E_e_ Gg B_b_ cc'
dd' dd' e_e_' e_e_'
dd' dd' cc' cc'
Gg Gg B_b_ B_b_ 
Gg Ff E_e_ Cc

.. .. .. ..

A,. D. E. D.
C. F. G. F.
A,. D. E. D.
C. F. G. F.
C. F. G. F.
E. A. d. A.
C. F. G. F.
E. A. d. A.

A,. D. E. D.
C. F. G. F.
A,. D. E. D.
C. F. G. F.
C. F. G. F.
E. A. d. A.
C. F. G. F.
E. A. d. A.

A. A. A. A.
G. G. F. F.
A. A. G. G.
F. F. E. E.
A. A. A. A.
G. G. F. F.
E. E. C. C.
A,. A,. C. C.




$
.. .. .. ..

c. .. .. B.
c. .. .. d.
e_. .. .. d.
e_. .. .. g.
c. .. .. B.
c. .. .. d.
e_. .. .. f.
g. .. .. ..

c. .. .. B.
c. .. .. d.
e_. .. .. d.
e_. .. .. f.
c. .. .. B.
c. .. .. d.
f. .. .. g.
c'. .. .. ..

c'. c'. g. g.
a_. a_. e_. e_.
d. e_. f. g.
e_. g. b_. c'.
g. g. a_. a_.
g. g. f. f.
c. c. d. d.
c. d. e_. g.

c'. c'. g. g.
a_. a_. e_. e_.
d. e_. f. g.
e_. g. b_. c'.
d'. d'. e_'. e_'.
d'. d'. c'. c'.
g. g. b_. b_.
g. f. e_. c.

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

g. g. d. d.
e_. e_. B_. B_.
A. B_. c. d.
B_. d. f. g.
d. d. e_. e_.
d. d. c. c.
G. G. A. A.
G. A. B_. d.

gG gG dD dD
eE_ ee_ B_B_, B_B_,
AA, B_B_, cC dD
B_B_, dD fF gG
aA aA b_B_ b_B_
aA aA gG gG
d'd d'd ff ff
d'd c'c b_B_ gG

.. .. .. ..

c. Bc d. cd
e. de f. ef
g. fe d. cB
A. GA Bc d.
c. Bc d. cd
e. de f. ef
g. fe d. cB
A. GA Bc d.

gf ed .. c.
gf ed .. c.
c'b ag .. a.
gf ed .. f.
gf ed .. c.
gf ed .. c.
c'b ag .. a.
gf ed .. f.

c. Bc d. cd
e. de f. ef
g. fe d. cB
A. .. G. ..
c. Bc d. cd
e. de f. ef
g. fe d. ce
c. .. c'. ..

$
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. .. 

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..

A,A#, B,C .. ..
C#D D#E .. ..
FF# GG# .. ..
AA# Bc .. ..
c#d d#e .. ..
ff# gg# .. ..
aa# bc' .. ..
c#'d' d#'e' .. ..

A,A#, B,C e'd#' d'c#'
C#D D#E c'b a#a
FF# GG# g#g f#f
AA# Bc e#d d#c
c#d d#e cB A#A
ff# gg# G#G F#F
aa# bc' ED# DC#
c#'d' d#'e' CB, A#, A,



$
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. c. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. c. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. g. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. g. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. c'. .. 
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. .. 
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. c'. .. 

.. .. c'. .. 
.. .. c'. .. 
.. .. c'. .. 
.. .. c'. .. 
.. .. c'. .. 
.. .. c'. .. 
.. .. c'. .. 
.. .. c'. .. 

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. .. 
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. .. 

.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. g. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. g. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. g. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. g. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. g. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. e'. ..

.. .. .. ..

.. .. .. c.
.. .. .. c.
.. .. .. c.
.. .. .. c.
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. c. .. ..
.. c. .. ..
.. c. .. ..
.. g. .. ..
.. .. .. g.
.. .. .. c.
.. .. .. c.
.. .. .. c.

... ... ... A..
... ... ... c..
... ... ... e..
... ... ... c'..
... d.. ... ...
... a.. ... ...
... f.. ... ...
... A.. ... ...

$
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. G. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. G. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. C. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. C. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. .. 
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. C. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. C. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

C. .. .. ..
C. .. .. ..
C. .. .. ..
C. .. .. ..
C. .. .. ..
C. .. .. ..
B_,. .. .. ..
B_,. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..

.. .. .. ..
.. .. C. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. C. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..
.. .. C. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. C. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..
.. .. C. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. C. ..
.. .. .. ..
.. .. .. ..

.. .. .. ..

.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. ..
.. .. .. c.
.. .. .. c.
.. .. .. c.
.. .. .. c.

.. .. .. c.
.. .. .. c.
.. .. .. c.
.. .. .. c.
.. c. .. ..
.. c. .. ..
.. c. .. ..
.. c. .. ..

... c.. ... ...
... e.. ... ...
... g.. ... ...
... b.. ... ...
... ... ... g..
... ... ... e..
... ... ... d..
... ... ... c..

`








